@@ -475,8 +475,13 @@ static int matchromaji(char *romaji, XChar2b *kstring)
 {
 	while(*romaji!='\0')
 	{
-		if(kstring->byte1 > 0x23){
-			return 0;
+		switch(kstring->byte1){
+			case 0x21:
+			case 0x22:
+			case 0x23:
+				break;
+			default:
+				return 0;
 		}
 		/*durn non-ascii punctuation codes...*/
 		if(kstring->byte1==0x21){
@@ -493,6 +498,7 @@ static int matchromaji(char *romaji, XChar2b *kstring)
 		romaji++;
 		kstring++;
 	}
+
 	return 1;
 }
 /* I'm throwing out the old romajitokana, and coding this more like
