@@ -21,12 +21,9 @@ extern void Beep();	/* nice generic routine that respects whether
 			 * player wants quiet or not
 			 */
 
+/* in mainwindow.c, and we dont have mainwindow.h */
+extern void ClearCheat();
 
-/* general-purpose widget operators.. */
-extern void SetWhiteOnBlack(Widget);
-extern void SetBlackOnWhite(Widget);
-extern void SetWidgetNumberval(Widget,int);
-extern void SetWidgetHexval(Widget,int);
 
 /* other "usefull" general functions */
 extern int xtoi(char *);
