@@ -376,6 +376,7 @@ void guessvalue(int guess)
 	kanjiP = values[guess];/* for setting "incorrect" flag */
 
 	if(guess == truevalue){
+		ClearCheat();
 		setstatus("Correct! Now try THIS one...");
 		switch(kanjiP->incorrect)
 		{
@@ -529,7 +530,11 @@ void printallchoices()
 
 	for(i=0;i<NUMBEROFCHOICES;i++)
 	{
-		SetBlackOnWhite(choicesWidgets[i]);
+		/* Used to call UnhighlightButton() on each, to make sure
+		 * 'cheat' effect went away.
+		 * Now, guessvalue() calls ClearCheat() if correct
+		 * value has been clicked on.
+		 */
 
 		switch(choicesmode)
 		{
