@@ -12,21 +12,26 @@ extern  TRANSLATION values[NUMBEROFCHOICES];
 
 extern void CountKanji();
 extern void SetupGuess();
+extern void TallyWrong();
+extern void ClearMissed();
 
 extern TRANSLATION pickkanji(); /* only for use from badguess.c */
 
 extern void BackCallback(Widget,XtPointer,XtPointer);
 extern void ChangeMode(Widget,XtPointer,XtPointer);
+extern void ChangeQuestion(Widget,XtPointer,XtPointer);
 extern void Guessvalue(Widget,XEvent *,String *,Cardinal *);
 extern void choicescallback(Widget,XtPointer,XtPointer),
 	    cheatcallback(Widget,XtPointer,XtPointer);
 
-extern void printenglish(Widget,TRANSLATION);	/* set english button widgets */
-extern void printkanji(Widget,TRANSLATION);	/* set english button widgets */
+extern void setenglabel(Widget,TRANSLATION);	/* set english button widgets */
+extern void setkanjilabel(Widget,TRANSLATION);	/* set english button widgets */
+extern void setkanalabel(Widget,TRANSLATION);	/* set english button widgets */
 extern void printallchoices(), printquestion();
 
-
+extern TRANSLATION picktruevalue();
 extern void guessvalue(int);
+extern void supercheat();
 
 extern Boolean UseThisKanji(int kanjinum);
 
