@@ -24,26 +24,11 @@ int gradelevelflags;
 
 
 
-/*
- * Sets widget to have white text on a black background
- */
-void SetWhiteOnBlack(Widget w){
-	XtVaSetValues(w,XtNbackground,black,
-		       XtNforeground,white,
-		       NULL);
-}
-/*
- * Sets widget to have black text on a white background
- */
-void SetBlackOnWhite(Widget w){
-	XtVaSetValues(w,XtNbackground,white,
-		       XtNforeground,black,
-		       NULL);
-}
-
 /* printgrades:
  *	update grade widgets, cause the stupid things don't seem to want
  *	to start as they SHOULD start. Grrr...
+ *	CANNOT CALL THIS from a grade button callback, due to
+ *	BUG in Xaw.so.6
  */
 
 void printgrades(){
@@ -137,7 +122,7 @@ void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 						return;
 						
 					}
-					SetBlackOnWhite(w);
+					ReverseButton(w);
 					if((int) data <=6)
 						sprintf(statusprefix,"Grade %d unset: ",(int) data);
 					else
@@ -146,7 +131,7 @@ void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 				/* CountKanji already called above */
                    	} else {
 			/* nope.. turn on grade level */
-				SetWhiteOnBlack(w);
+				ReverseButton(w);
 				gradelevelflags |= grademask;
 					if((int) data <=6)
 						sprintf(statusprefix,"Grade %d set: ",(int) data);
