@@ -326,8 +326,8 @@ void initstuffs(int *argc,char *argv[])
 	 * later
 	 */
 
-	white = WhitePixel(display,0);
-	black = BlackPixel(display,0);
+	white = XWhitePixel(display,0);
+	black = XBlackPixel(display,0);
 
 	GetOptions();
 
