@@ -155,30 +155,6 @@ XChar2b std_translations[NUM_OF_KW][2]=
 
 XChar2b paragraphglyph[2] = { {0x22, 0x79}, {0x0, 0x0} };
 
-/* I could really use this as a better way to do translations in
- * kanatoromaji. sigh
- */
-static char * romaji_translations[NUM_OF_KW]=
-{
-	"A",   "I",   "U",   "E",   "O",
-	"KA",  "KI",  "KU",  "KE",  "KO",
-	"SA",  "SHI", "SU",  "SE",  "SO",
-	"TA",  "CHI", "TSU", "TE",  "TO",
-	"NA",  "NI",  "NU",  "NE",  "NO",
-	"HA",  "HI",  "FU",  "HE",  "HO",
-	"MA",  "MI",  "MU",  "ME",  "MO",
-	"RA",  "RI",  "RU",  "RE",  "RO",
-	"GA",  "GI",  "GU",  "GE",  "GO",
-	"ZA",  "ZI",  "ZU",  "ZE",  "ZO",
-	"DA",  "DI",  "DU",  "DE",  "DO",
-	"BA",  "BI",  "BU",  "BE",  "BO",
-	"PA",  "PI",  "PU",  "PE",  "PO",
-	"YA",  "",    "YU",  "",    "YO",
-	"ya",  "",    "yu",  "",    "yo",
-	"WA",  "wa",  "tsu", "N",   "O"
-};
-
-
 /************************************************************/
 
 /*
@@ -341,6 +317,10 @@ void ksearch_toggles(Widget w, XtPointer data, XtPointer call_data)
 
 		for(kcount=0; kcount<80; kcount++)
 		{
+			/* skip symbol type buttons */
+			if(std_translations[kcount][0].byte1!=0x24)
+				continue;
+
 			kstr[0].byte2 = std_translations[kcount][0].byte2;
 
 			XtVaSetValues(kanawidgets[kcount],
@@ -447,7 +427,7 @@ void makeinputwidgets(Widget parent)
 			XtNwidth, 150,
 			XtNfromVert, searchbutton,
 			XtNencoding, XawTextEncodingChar2b,
-			XtNstring,"\0\0",
+			XtNlabel,"\0\0",
 			XtNfont, smallkfont,
 			XtNcursor, textcursor,
 			NULL);
