@@ -64,9 +64,9 @@ static void selectquarter(Widget w, XtPointer data, XtPointer call_data)
 	for(kcount=0; kcount<4; kcount++)
 	{
 		if(num == kcount)
-			ReverseButton(kanjiquarters[kcount]);
+			HighlightButton(kanjiquarters[kcount]);
 		else
-			UnreverseButton(kanjiquarters[kcount]);
+			UnhighlightButton(kanjiquarters[kcount]);
 	}
 	quarter_up = num;
 	
