@@ -21,6 +21,10 @@
 #include "utils.h"
 #include "externs.h"
 
+/* Keeps track of if we are using 'cheat' to have a button highlighted */
+
+static cheat_on=0;
+
 /* BackCallback:
  *	go back to last kanji.
  *  Has to be in game.c, to access static vars
@@ -129,14 +133,17 @@ void Guessvalue(Widget w,XEvent *event,String *params,Cardinal *num_parags){
 
 /* This is called by both button and key-accelerator */
 /* set client_data to non-NULL if you dont want status message set */
-void cheatcallback(Widget w,XtPointer client_data,XtPointer calldata){
-
+void cheatcallback(Widget w,XtPointer client_data,XtPointer calldata)
+{
 	ReverseButton(choicesWidgets[truevalue]);
+	cheat_on=!cheat_on;
+	printallchoices();
 
-	if(client_data ==NULL){
+	if((client_data ==NULL) && cheat_on){
 		setstatus("Try Harder!");
 	}
 }
+
 /* This is currently ONLY called by key-accelerator
  * Do BOTH highlight of correct value, AND pop up search window
  * with full details of that kanji
@@ -146,6 +153,13 @@ void supercheat(){
 	printsearch(values[truevalue]);
 }
 
+/* Hook for guessvalue in game.c to clear 'cheat' highlighting */
+void ClearCheat(){
+	if(!cheat_on) return;
+	ReverseButton(choicesWidgets[truevalue]);
+	
+}
+
 /* choicescallback:
  *	Handles clicking on guess
  */
