@@ -48,8 +48,11 @@
 /* This is the maximum translations we will store up and display */
 #define MAXTRANS 200
 
-static TRANSLATION transptrs[MAXTRANS];
-static Widget transwin[MAXTRANS]; /* user clicks to select translation*/
+
+static Widget      transwin[MAXTRANS];       /* user clicks to select translation*/
+static TRANSLATION transptrs[MAXTRANS];	     /* info in each transwin */
+static Widget      usefiletoggles[MAXTRANS]; /* is each one in usefile? */
+
 static int transcount=0;
 static int overflow=0; /* number of added translations over MAXTRANS */
 
@@ -111,29 +114,21 @@ static void setmultistatus(char *string){
 }
 
 
-
-#define CLEARALL
-
-
 void ClearAllMulti(){
 	int tc;
 
 	overflow=0;
 
 	for(tc=0; tc<transcount; tc++){
-#ifdef CLEARALL
+
 		XtDestroyWidget(transwin[tc]);
-#else
-		XtVaSetValues(transwin[tc],
-			      XtNlabel,"",
-			      NULL);
-#endif
+		XtDestroyWidget(usefiletoggles[tc]);
 	}
 
-#ifdef CLEARALL
 	bzero(transptrs, sizeof(transptrs));
 	bzero(transwin, sizeof(transwin));
-#endif
+	bzero(usefiletoggles, sizeof(usefiletoggles));
+
 	transcount=0;
 	setmultistatus("No matches");
 }
@@ -183,6 +178,20 @@ void multicallback(Widget button, XtPointer data, XtPointer call_data){
 	
 }
 
+/* This handles a click on a usefile toggle button, 
+ * in the multikanji popup.
+ */
+void multiUcallback(Widget button, XtPointer data, XtPointer call_data){
+	Pixel fg,bg;
+	int kindex=(int)data;
+
+	SetUseKanji(kindex,!InUsefile(kindex));
+	ReverseButton(button);
+
+	/* Just in case this is for the currently displayed search line*/
+	updatesearchlabels();
+
+}
 
 #define VWIDTH 350
 #define VHEIGHT 300
@@ -318,13 +327,18 @@ void setmultilabels(){
 
 /* We ignore attempts to add more than our MAXTRANS value */
 void AddMultiTranslation(TRANSLATION nt){
-	Widget newbutton;
+	int kindex;
+
+	Widget newbutton,newu;
 	Widget prevbutton=NULL;
 	int formwidth=GetWidgetWidth(multi_popup);
 	char statusprompt[20];
-	char buttname[10];
+	char buttname[10],uname[10];
+
+	kindex=trans_to_index(nt);
 
 	sprintf(buttname, "multi%x\n", (unsigned int)nt);
+	sprintf(uname, "multiU%x\n", (unsigned int)nt);
 
 	if(formwidth>50)
 		formwidth-=50;
@@ -342,8 +356,6 @@ void AddMultiTranslation(TRANSLATION nt){
 
 	transptrs[transcount]=nt;
 
-#ifdef CLEARALL
-
 	if(transcount>0){
 		prevbutton=transwin[transcount-1];
 	}
@@ -362,7 +374,29 @@ void AddMultiTranslation(TRANSLATION nt){
 
 	XtAddCallback(newbutton, XtNcallback, multicallback, NULL);
 	transwin[transcount]=newbutton;
-#endif /* CLEARALL*/
+
+	newu=XtVaCreateManagedWidget(buttname, commandWidgetClass,
+			multi_form,
+			XtNfromVert,prevbutton,
+			XtNfromHoriz,newbutton,
+			XtNwidth, formwidth,
+			/*
+			XtNencoding,XawTextEncodingChar2b,
+			XtNfont,largekfont,
+			XtNlabel,Ulabel,
+			*/
+			XtNencoding,XawTextEncoding8bit,
+			XtNfont,englishfont,
+			XtNlabel,"u",
+			NULL);
+
+	XtAddCallback(newu, XtNcallback, multiUcallback, (XtPointer)kindex);
+
+	if(InUsefile(kindex)){
+		ReverseButton(newu);
+	}
+
+	usefiletoggles[transcount]=newu;
 
 	switch(displaymode){
 	   case GUESS_ENGLISH:
