@@ -518,6 +518,7 @@ void readedict()
 		}
 
 		/* Success! Set pointers appropriately */
+		newk->kdrill_index=nextindex;
 		translations[nextindex++] = newk;
 		if(lastk != NULL)
 		{
@@ -794,6 +795,7 @@ void readstructs(){
 		}
 
 		StripBrackets(newk->english, parse);
+		newk->kdrill_index=Kanji;
 		translations[Kanji] = newk;
 		numberofkanji++;
 		if(numberofkanji%1000 == 0)
