@@ -21,6 +21,8 @@
 #include "multikanji.h"
 #include "strokesearch.h"
 #include "readfile.h"
+#include "utils.h"
+#include "log.h"
 
 #include "searchwidgets.h"
 
@@ -59,7 +61,7 @@ void dostrokefind(BYTE strokecount, BITS16 skipval)
 				continue;
 			}
 
-		AddTranslation(searchtarget);
+		AddMultiTranslation(searchtarget);
 		if(firstmatch==NULL)
 			firstmatch=searchtarget;
 
@@ -67,7 +69,9 @@ void dostrokefind(BYTE strokecount, BITS16 skipval)
 	}
 
 	if(firstmatch!=NULL){
-		ShowMulti();
+		if(getMultiCount() >1){
+			ShowMulti();
+		}
 	} else{
 		setstatus("No match found");
 		Beep();
@@ -75,6 +79,25 @@ void dostrokefind(BYTE strokecount, BITS16 skipval)
 	
 }
 
+/* called by doenglishfind, if we need to restrict search to
+ * "beginning of word only"
+ * return 1 on match, 0 on not.
+ */
+int matchengfirst(char *full, char *frag){
+	int len=strlen(frag);
+	while(1){
+		if(strncmp(full,frag,len) ==0){
+			return 1;
+		}
+		full=strchr(full,':'); /* actually, " : " */
+		if(full==NULL){
+			return 0;
+		}
+		full+=2;
+	}
+	return 0;
+}
+
 /* doenglishfind:
  *	Assume what we want to look for is in the search ENGLISH INPUT widget.
  *	First, read in that value from the widget
@@ -102,18 +125,32 @@ TRANSLATION doenglishfind()
 
 	searchtarget=translations[lowestkanji];
 	ClearAllMulti();
-	while(searchtarget!=NULL){
-		if(strstr(searchtarget->english,str) != NULL){
-			AddTranslation(searchtarget);
-			if(firstmatch==NULL)
-				firstmatch=searchtarget;
+	if(match_onlyatstart){
+		while(searchtarget!=NULL){
+			if(matchengfirst(searchtarget->english,str) == 1){
+				AddMultiTranslation(searchtarget);
+				if(firstmatch==NULL)
+					firstmatch=searchtarget;
 			
+			}
+			searchtarget = searchtarget->nextk;
+		}
+	} else {
+		while(searchtarget!=NULL){
+			if(strstr(searchtarget->english,str) != NULL){
+				AddMultiTranslation(searchtarget);
+				if(firstmatch==NULL)
+					firstmatch=searchtarget;
+			
+			}
+			searchtarget = searchtarget->nextk;
 		}
-		searchtarget = searchtarget->nextk;
 	}
 
 	if(firstmatch!=NULL){
-		ShowMulti();
+		if(getMultiCount() >1){
+			ShowMulti();
+		}
 	} else{
 		setstatus("No match found");
 		Beep();
@@ -123,17 +160,43 @@ TRANSLATION doenglishfind()
 	
 }
 
+/********************************************************************
+ * suite of really stupid-looking routines, so I can do INTELLIGENT *
+ * generic-comparisons in DoFind, and posibly elsewhere             *
+ ********************************************************************/
+
+Boolean compareUnicode(TRANSLATION entry, int compvalue){
+	if(entry->Uindex==compvalue) return True;
+	return False;
+}
+Boolean compareHalpern(TRANSLATION entry, int compvalue){
+	if(entry->Hindex==compvalue) return True;
+	return False;
+}
+Boolean compareNelson(TRANSLATION entry, int compvalue){
+	if(entry->Nindex==compvalue) return True;
+	return False;
+}
+Boolean compareFreq(TRANSLATION entry, int compvalue){
+	if(entry->frequency==compvalue) return True;
+	return False;
+}
+
+/************************************************************/
+
+
 /* DoFind
  *   (do-find)accelerator hook
  *   This handles callbacks for multiple widgets.
- *
- *	Farms out english search to doenglishearch().
- *	Also handles <return> press for numerical input windows
- *	on the search popup. eg "F:",
+ *   Basically, any inputfield on the Search window, that uses
+ *   [ENTER] as a trigger for a search.
+ *   Except for the kanasearch
  */
 void 
 DoFind(Widget w,XEvent *event,String *params,Cardinal *num_parags){
 	TRANSLATION findtarget = NULL;
+	int targetval=0, retval=0, ndx=0;
+	Boolean (* comparefunc)(TRANSLATION,int);
 
 	setstatus("Searching...");
 
@@ -148,14 +211,16 @@ DoFind(Widget w,XEvent *event,String *params,Cardinal *num_parags){
 			printsearch(findtarget);
 		}
 		return;
-		
 	}
 
+	/* Special case: not a "find this index", but a
+	 * "find me this index or greater"
+	 */
 	if (w == searchnumbers[POUND_INPUT])
 	{
 		int number;
 		/* want closest index match */
-		number = GetWidgetNumberval(w);
+		number = GetWidgetHexval(w);
 
 		if((number <lowestkanji) || (number> highestkanji)){
 			setstatus("input out of dict. range");
@@ -181,74 +246,53 @@ DoFind(Widget w,XEvent *event,String *params,Cardinal *num_parags){
 		findtarget = translations[number];
 
 		printsearch(findtarget);
-	} else 	if (w == searchnumbers[F_INPUT])
-	{
-		/* want closest frequency match */
-		/* "F:" input widget */
-		int findfreq=-1;
-		TRANSLATION cur_target = translations[lowestkanji];
-		
 
-		findfreq = GetWidgetNumberval(w);
-
-		if(findfreq <1){
-			setstatus("frequencies must be greater than 0");
-			Beep();
-			printsearch(lastsearch);
-			return;			
-		}
-
-		cur_target = translations[lowestkanji];
-		findtarget = cur_target;
-
-		while(cur_target != NULL){
-			if(cur_target->frequency == findfreq){
-				findtarget = cur_target;
-				break;
-			}
-			if((findfreq - (cur_target->frequency)) <
-			   (findfreq - (findtarget->frequency)) ){
-				findtarget = cur_target;
-			}
-			cur_target = cur_target->nextk;
-			   
-		}
-		
-		printsearch(findtarget);
-
-	} else {
-		/* find exact match for special index */
-		char append='\0';
-		char findstring[100];
-		String str;
-
-		
-		if(w == searchnumbers[H_INPUT]){
-			append = 'H';
-		} else if(w == searchnumbers[N_INPUT]){
-			append = 'N';
-		} else if(w == searchnumbers[U_INPUT]){
-			append = 'U';
-		} else {
-			puts("Where did that input come from??");
-			return;
-		}
-		
-		XtVaGetValues(w,XtNstring,&str,NULL);
-
-		sprintf(findstring,"%c%.99s",append,str);
-
-		/* FindIndex sets status appropriately on error */
-
-		findtarget = translations[FindIndex(findstring)];
-		printsearch(findtarget);
-		
+		return;
 	}
+
+
+	if(w==searchnumbers[U_INPUT]) {
+		targetval=GetWidgetHexval(w);
+	} else {
+		targetval=GetWidgetNumberval(w);
+	}
+
+	if(w==searchnumbers[U_INPUT]){
+		comparefunc=compareUnicode;
+	} else if (w == searchnumbers[F_INPUT]) {
+		comparefunc=compareFreq;
+	} else if (w == searchnumbers[H_INPUT]) {
+		comparefunc=compareHalpern;
+	} else if (w == searchnumbers[N_INPUT]) {
+		comparefunc=compareNelson;
+	} else {
+		puts("DEBUG: ERROR: unrecognized compare type in DoFind");
+		return ;
+	}
+
+	for(ndx=lowestkanji;ndx<highestkanji;ndx++){
+		if(translations[ndx]==NULL)
+			continue;
+		if(comparefunc(translations[ndx],targetval)){
+			retval=ndx;
+			break;
+		}
+	}
+
+
+	if(retval!=0){
+		findtarget = translations[retval];
+		printsearch(findtarget);
+		setstatus("Displaying match");
+	} else {
+		setstatus("No match found");
+		Beep();
+	}
+
 	
 
 }
 
-
 /* if fragment matches FIRST chars of fullstring, return 1.
  * Otherwise, return  0
  *   Allow hiragana/kanakana folding.
@@ -290,13 +334,13 @@ int matchkanafirst(XChar2b *fullstring, XChar2b *fragment)
  *	return 1 on match, 0 on fail.
  *
  *	Note that we try to allow match of kanagana, by hiragana.
- *	If kana_onlyatstart!=0, only match if start of a kana phrase
+ *	If match_onlyatstart!=0, only match if start of a kana phrase
  *	matches fragment
  */
 int matchkana(XChar2b *fullstring, XChar2b *fragment)
 {
 
-	if(kana_onlyatstart)
+	if(match_onlyatstart)
 	{
 		while(fullstring->byte1 != 0)
 		{
@@ -357,7 +401,7 @@ int matchkana(XChar2b *fullstring, XChar2b *fragment)
  *	Equivalent to matchkana!
  *	return 1 if match, 0 if no match
  *
- *	Note that this takes intoaccount 'kana_onlyatstart'
+ *	Note that this takes intoaccount 'match_onlyatstart'
  *	Which will presumably only match the kanji if at beginning
  *	of the phrase. Hmmm.
  */
@@ -383,7 +427,7 @@ void dokanafind(XChar2b *target)
 
 	while(searchtarget != NULL){
 		if(matchkana(searchtarget->pronunciation, target)){
-			AddTranslation(searchtarget);
+			AddMultiTranslation(searchtarget);
 			if(firstmatch==NULL)
 				firstmatch=searchtarget;
 		}
@@ -392,7 +436,9 @@ void dokanafind(XChar2b *target)
 
 	if(firstmatch!=NULL){
 		printsearch(firstmatch);
-		ShowMulti();
+		if(getMultiCount() >1){
+			ShowMulti();
+		}
 	} else{
 		setstatus("No match found");
 		Beep();
@@ -401,6 +447,38 @@ void dokanafind(XChar2b *target)
 	return;
 }
 
+
+/* This is here, because I want to preserve all "search" type
+ * routines together in this file.
+ * But it is called by Handle_showusefile
+ *
+ * The purpose is to let the user easily see which chars are in the
+ * usefile, and easily remove them. They already have an easy way
+ * to ADD them, via the search window :-)
+ */
+void dousefilefind()
+{
+	/*int kcount=getMultiMax();*/
+	int kindex=lowestkanji;
+
+	ClearAllMulti();
+	for(kindex=lowestkanji; kindex<=highestkanji;kindex++){
+		if(InUsefile(kindex)){
+			AddMultiTranslation(translations[kindex]);
+
+			/*
+			 * let the multi-window complain about
+			 * truncation, if appropriate
+			if(--kcount <=0){
+				return;
+			}
+			*/
+		}
+	}
+
+	ShowMulti();
+}
+
 /* dokanjifind()
  *	find match to "Four Corner" encoding in kanjidic translations
  */
@@ -417,7 +495,7 @@ void dokanjifind(int target)
 
 	while(searchtarget != NULL){
 		if(searchtarget->Qindex == target){
-			AddTranslation(searchtarget);
+			AddMultiTranslation(searchtarget);
 			if(firstmatch==NULL)
 				firstmatch=searchtarget;
 		}
@@ -428,7 +506,9 @@ void dokanjifind(int target)
 
 	if(firstmatch!=NULL){
 		printsearch(firstmatch);
-		ShowMulti();
+		if(getMultiCount() >1){
+			ShowMulti();
+		}
 	} else{
 		setstatus("No match found");
 		Beep();
@@ -454,7 +534,7 @@ void doskipfind(int target)
 
 	while(searchtarget != NULL){
 		if(searchtarget->Sindex == target){
-			AddTranslation(searchtarget);
+			AddMultiTranslation(searchtarget);
 			if(firstmatch==NULL)
 				firstmatch=searchtarget;
 		}
@@ -465,7 +545,9 @@ void doskipfind(int target)
 
 	if(firstmatch!=NULL){
 		printsearch(firstmatch);
-		ShowMulti();
+		if(getMultiCount() >1){
+			ShowMulti();
+		}
 	} else{
 		setstatus("No match found");
 		Beep();
@@ -548,6 +630,16 @@ void process_kinput(XChar2b in_char)
 	
 }
 
+
+void debugkprint(XChar2b* kstr){
+	XChar2b *ptr=kstr;
+	while(ptr->byte1 != 0){
+		printf("%x%x",ptr[0].byte1,ptr[0].byte2);
+		ptr++;
+	}
+	printf("\n");
+}
+
 /****************************************************************\
  * Quickie to find edict matches for a kanji string             *
  * Will also find assumed original match in kanjidic,		*
@@ -570,14 +662,19 @@ void findkanjiall(XChar2b *Schars)
 				init=1;
 				printsearch(searchparse);
 			}
-			AddTranslation(searchparse);
+			AddMultiTranslation(searchparse);
 		}
 		searchparse=searchparse->nextk;
 	}
 
 	if(init>0)
 	{
-		ShowMulti();
+		/* Search window already updated. So only show "multi"
+		 * window, if there are actual multiple matches
+		 */
+		if(getMultiCount() >1){
+			ShowMulti();
+		}
 	} else {
 		setstatus("no match found");
 	}
