@@ -6,7 +6,10 @@ extern XChar2b std_translations[NUM_OF_KW][2];
 
 extern TRANSLATION lastsearch;	/* holds index of last kanji searched */
 
+/* Finds all matchs to a particular kanji string */
 extern void findkanjiall(XChar2b *);
 
 extern void dostrokefind(BYTE strokecount, BITS16 skipval);
 extern TRANSLATION doenglishfind();
+extern void dousefilefind();
+extern void doskipfind(int target);
