@@ -129,18 +129,7 @@ void Handle_matchkanji(Widget w,XtPointer client_data, XtPointer call_data)
  */
 int trans_to_index(TRANSLATION trans)
 {
-	int kcount=lowestkanji;
-	for(;kcount<=highestkanji;kcount++)
-	{
-		if(translations[kcount] == trans)
-			return kcount;
-	};
-
-	if(kcount>highestkanji)
-	{
-		puts("ERROR: cannot find index for translations[] ?!!");
-	}
-	return kcount;
+	return trans->kdrill_index;
 }
 
 /* handle toggle for "This kanji in in usefile" button*/
@@ -150,13 +139,6 @@ void Handle_usetoggle(Widget w,XtPointer client_data, XtPointer call_data)
 	int kcount=trans_to_index(lastsearch);
 	SetUseKanji(kcount, !InUsefile(kcount));
 	CountKanji();
-	
-	XtVaGetValues(w, XtNforeground, &fg,
-		      XtNbackground, &bg, NULL);
-
-	XtVaSetValues(w, XtNforeground, bg,
-		      XtNbackground, fg, NULL);
-		    
 }
 
 /* handle "Show" usefile button */
@@ -193,10 +175,10 @@ void updatesearchlabels()
 
 	if(InUsefile(kindex))
 	{
-		ReverseButton(searchnumbers[INUSEFILE]);
+		HighlightButton(searchnumbers[INUSEFILE]);
 	} else
 	{
-		UnreverseButton(searchnumbers[INUSEFILE]);
+		UnhighlightButton(searchnumbers[INUSEFILE]);
 	}
 
 	/* assume if kanji entry is ONE char long, that we
@@ -369,13 +351,17 @@ void Handle_searchPasted(Widget widget, XtPointer cl, XEvent *e, Boolean *cont)
 
 	strparse=0;
 
+	if(str[strparse]=='\0'){
+		doFunkyGrab(widget);
+	}
+#define OLDDEBUG
 #ifdef OLDDEBUG
-	puts("values of XtNstring pasted are:");
-	do
+	puts("Handle_searchPasted: values of XtNstring pasted are:");
+	while(str[strparse] != '\0');
 	{
-		printf("%.2x ",str[strparse]);
+		printf("%.2x.",str[strparse++]);
 
-	} while(str[strparse++] != '\0');
+	} 
 	puts("");
 	printf("strparse is now %d\n", strparse);
 #else
