@@ -144,11 +144,11 @@ void SKIPcallbacks(Widget w,XtPointer client_data, XtPointer call_data)
 	{
 		if(scount==skipnum)
 		{
-			ReverseButton(SKIPcategories[scount]);
+			HighlightButton(SKIPcategories[scount]);
 		}
 		else
 		{
-			UnreverseButton(SKIPcategories[scount]);
+			UnhighlightButton(SKIPcategories[scount]);
 		}
 	}
 	setSKIPlabels(skipnum);
