@@ -64,10 +64,10 @@ void StrokeSKIPcount(Widget w, XtPointer data, XtPointer call_data)
 {
 	int bcount;
 	for(bcount=0;bcount<SKIP_NUMW;bcount++){
-		UnreverseButton(skiptoggle[bcount]);
+		UnhighlightButton(skiptoggle[bcount]);
 	}
 
-	ReverseButton(w);
+	HighlightButton(w);
 	if(w == skiptoggle[SKIP_RL]){
 		skipval=skipfromthree(1,0,0);
 	}
