@@ -8,6 +8,7 @@
 #include <Xaw/Command.h>
 #include <Xaw/Label.h>
 #include <Xaw/Form.h>
+#include <Xaw/Paned.h>
 #include <Xaw/Box.h>
 #include <Xaw/AsciiText.h>
 
@@ -17,8 +18,13 @@
 #include "grades.h"
 #include "options.h"
 #include "search.h"
-
+#include "searchwidgets.h"
+#include "learn.h"
+#include "utils.h"
+#include "multikanji.h"
+#include "init.h"
 #include "log.h"
+#include "timeout.h"
 
 Widget toplevel,form;
 
@@ -45,12 +51,23 @@ static char * jumpAccel =
 /* setstatus:
  *  sets label for main status bar widget.
  *	( the long one at the bottom)
+ *  and also the same message at the bottom of the search popup window
  */
 void setstatus(char *s)
 {
+	if((statusline==NULL) || (searchstatusline==NULL)){
+		/* Some kind of warning message, before program has fullyh
+		 * come up. Print to stdout instead
+		 */
+		printf("%s\n", s);
+		return;
+	}
 	XtVaSetValues(statusline,
 		      XtNlabel,s,
 		      NULL);
+	XtVaSetValues(searchstatusline,
+		      XtNlabel,s,
+		      NULL);
 }
 
 
@@ -65,9 +82,12 @@ handle_button(Widget w,XtPointer closure,XEvent *e,Boolean *cont)
 
 	if(e->type == ClientMessage)
 		puts("CLIENT MESSAGE FROM BUTTONH");
-	if(event->button != Button2) return;
+	if((event->button != Button2) &&
+	   (event->button != Button3))
+		return;
+
 #ifdef DEBUG
-	puts("Got button2 click?");
+	puts("Got button2/3 click?");
 #endif
 
 	do {
@@ -215,6 +235,46 @@ void JumpToKanji(Widget w,XEvent *event,String *params,Cardinal *num_parags)
 	
 }
 
+/* This is a special hack to attempt to force all the widgets to be the 
+ * same size.
+ * It has the unfortunate side-effect of killing initial auto-size-width
+ * for the top-level main window.
+ * So if you use this, remember to do a good layout of the top-level window
+ * as well. SIGH.
+ *
+ * The standard creation methods like MakeQuestionWidget are set up to
+ * create the forms if they have not already been made here.
+ */
+void MakeSpecialForms(Widget parent){
+
+	/* We try to set resource so that we will be happy  with EITHER
+	 * a Paned parent, or a Form parent.
+	 */
+
+
+	kanjiform = XtVaCreateManagedWidget("kanjiform",formWidgetClass, form,
+			XtNshowGrip, False,
+			XtNresizeToPreferred, True,
+
+			NULL);
+	englishform = XtVaCreateManagedWidget("englishform",formWidgetClass,
+				form,
+		XtNfromVert,kanjiform,
+		XtNvertDistance,10,
+			XtNshowGrip, False,
+			XtNresizeToPreferred, True,
+				NULL);
+	buttonform = XtVaCreateManagedWidget("buttonform",formWidgetClass,form,
+		     XtNfromVert,englishform,
+		     XtNvertDistance,15,
+			XtNshowGrip, False,
+			XtNskipAdjust, True,
+					    NULL);
+
+}
+
+
+
 /* MakeCurrent;
  *	Makes window that describes current widget.
  *	Has current kanji grade, frequency, and index listed
@@ -280,16 +340,23 @@ Widget MakeCurrent(Widget parent,Widget fromvert){
 }
 
 
+static char *missedAccel = " \
+  <Key>BackSpace:	Accelerator(x)\n \
+  <Key>Delete:		Accelerator(x)\n \
+";
+
 /* MakeQuestionWidget()
  *	set up widgets for top section of main window
  */
 void MakeQuestionWidget(){
 	Widget tmplabel;
+	XtAccelerators Accel;
 
+	if(kanjiform == NULL){
 	kanjiform = XtVaCreateManagedWidget("kanjiform",formWidgetClass, form,
 			XtNright,XawChainRight,
 			NULL);
-
+	}
 
 	questionWidget = XtVaCreateManagedWidget("questionlarge",
 						  commandWidgetClass,
@@ -334,6 +401,8 @@ void MakeQuestionWidget(){
 			XtNdisplayCaret,False,
 			XtNvertDistance, 10,
 			NULL);
+	Accel = XtParseAcceleratorTable(missedAccel);
+	XtOverrideTranslations(kanjiMissed,Accel);
 
 }
 
@@ -345,11 +414,14 @@ void MakeChoicesButtons(){
 	int i;
 
 
+	if(englishform==NULL) {
 	englishform = XtVaCreateManagedWidget("englishform",formWidgetClass,
 				form,
 				XtNfromVert,kanjiform,
-				XtNvertDistance,10,
+				XtNvertDistance,5,
 				NULL);
+	}
+
 	for(i=0;i<NUMBEROFCHOICES;i++){
 		char labl[20];
 		sprintf(labl,"english%d",i);
@@ -361,6 +433,7 @@ void MakeChoicesButtons(){
 			XtNright,XawChainRight,
 			XtNleft,XawChainLeft,
 			XtNfont,largekfont,
+			XtNvertDistance,10,
 			XtNshapeStyle,XawShapeRoundedRectangle,
 			XtNcornerRoundPercent,50,
 			NULL);
@@ -376,7 +449,8 @@ void MakeChoicesButtons(){
 			XtNcornerRoundPercent,50,
 			NULL);
 		}
-		XtAddEventHandler(choicesWidgets[i],ButtonPressMask,False,handle_button,NULL);
+		XtAddEventHandler(choicesWidgets[i],ButtonPressMask,False,
+				  handle_button,NULL);
 		XtAddCallback(choicesWidgets[i],
 			      XtNcallback,choicescallback,(XtPointer) i);
 
@@ -385,12 +459,14 @@ void MakeChoicesButtons(){
 }
 
 void MakeButtons(){
-	Widget w[6];
+	Widget w[7];
 
+	if(buttonform==NULL){
 	buttonform = XtVaCreateManagedWidget("buttonform",formWidgetClass,form,
 					     XtNfromVert,englishform,
 					     XtNvertDistance,15,
 					    NULL);
+	}
 
 	w[0] = XtVaCreateWidget("quit",commandWidgetClass,buttonform,
 			XtNlabel,"quit",
@@ -401,59 +477,67 @@ void MakeButtons(){
 			XtNfromHoriz,w[0],
 			NULL);
 
-	w[3]= XtVaCreateManagedWidget("back",commandWidgetClass,
+	w[2]= XtVaCreateManagedWidget("back",commandWidgetClass,
 			buttonform,
 			XtNlabel,"prev. char",
 			XtNfromHoriz,w[1],
-			XtNshapeStyle,XawShapeEllipse,
+			/*XtNshapeStyle,XawShapeEllipse,*/
 			NULL);
 
-	w[2] = XtVaCreateWidget("options",commandWidgetClass,
+	w[3]= XtVaCreateManagedWidget("log",commandWidgetClass,
+			buttonform,
+			XtNlabel,"Log",
+			XtNfromHoriz,w[2],
+			/*XtNshapeStyle,XawShapeEllipse,*/
+			NULL);
+
+	w[4] = XtVaCreateWidget("options",commandWidgetClass,
 			buttonform,
 			XtNlabel,"options",
 			XtNfromHoriz,w[3],
 			XtNshapeStyle,XawShapeEllipse,
 			NULL);
 
-	w[4]= XtVaCreateManagedWidget("search",commandWidgetClass,
+	w[5]= XtVaCreateManagedWidget("search",commandWidgetClass,
 			buttonform,
 			XtNlabel,"Search",
-			XtNfromHoriz,w[2],
-			XtNshapeStyle,XawShapeEllipse,
-			NULL);
-
-	w[5]= XtVaCreateManagedWidget("log",commandWidgetClass,
-			buttonform,
-			XtNlabel,"Log",
 			XtNfromHoriz,w[4],
 			XtNshapeStyle,XawShapeEllipse,
 			NULL);
+	w[6]= XtVaCreateManagedWidget("learn",commandWidgetClass,
+			buttonform,
+			XtNlabel,"Learn",
+			XtNfromHoriz,w[5],
+			XtNshapeStyle,XawShapeEllipse,
+			NULL);
+
 
 	statusline = XtVaCreateManagedWidget("status",labelWidgetClass,
 					     buttonform,
-		      XtNlabel,"Welcome to kanjidrill",
-		      XtNwidth,FULLWIDTH,
-		      XtNborderWidth,2,
-		      XtNvertDistance,STATUSDISTANCE,
-		      XtNfromVert,w[0],
-		      XtNresize,True,
+
+			XtNlabel,"             Welcome to kanjidrill             ",
+
+			XtNborderWidth,2,
+			XtNvertDistance,STATUSDISTANCE,
+			XtNfromVert,w[0],
 		      NULL);
 
 	XtAddCallback(w[0],XtNcallback,quit,NULL);
 	XtAddCallback(w[1],XtNcallback,cheatcallback,NULL);
-	XtAddCallback(w[2],XtNcallback,OptionsCallback,NULL);
-	XtAddCallback(w[3],XtNcallback,BackCallback,NULL);
-	XtAddCallback(w[4],XtNcallback,SearchCallback,NULL);
-	XtAddCallback(w[5],XtNcallback,MakeLog,NULL);
+	XtAddCallback(w[2],XtNcallback,BackCallback,NULL);
+	XtAddCallback(w[3],XtNcallback,MakeLog,NULL);
+	XtAddCallback(w[4],XtNcallback,OptionsCallback,NULL);
+	XtAddCallback(w[5],XtNcallback,SearchCallback,NULL);
+	XtAddCallback(w[6],XtNcallback,LearnCallback,NULL);
 	
-	XtManageChildren(w,5);
+	XtManageChildren(w,7);
 }
 
 
 void initfonts(){
 	char LargeKname[100],SmallKname[100],Efontname[100];
-	Widget tmplabel;
-	String tmpstring;
+/*	Widget tmplabel;*/
+/*	String tmpstring;*/
 
 	/* Sigh, okay this is ugly, tell me a better way...*/
 	/* Hell, tell me a way that WORKS, dammit... */
@@ -468,11 +552,11 @@ void initfonts(){
 	XtDestroyWidget(tmplabel);
 	*/
 	
-	GetXtString("kanjifont","Kanjifont",LargeKname);
-	GetXtString("smallkanji","Smallkanji",SmallKname);
-	GetXtString("englishfont","Englishfont",Efontname);
+	GetXtrmString("kanjifont","Kanjifont",LargeKname);
+	GetXtrmString("smallkanji","Smallkanji",SmallKname);
+	GetXtrmString("englishfont","Englishfont",Efontname);
 
-/*	GetXtString("font", "Font", tmpstring);*/
+/*	GetXtrmString("font", "Font", tmpstring);*/
 
 #ifdef DEBUG
 	printf("kanjifont from resources is \"%s\"\n",LargeKname);
@@ -500,30 +584,35 @@ void initfonts(){
 
 }
 
-
 static char *guessAccel = " \
   Ctrl<Key>q:	quit()\n \
-  None<Key>1:		guess-meaning(0)\n \
-  None<Key>2:		guess-meaning(1)\n \
-  None<Key>3:		guess-meaning(2)\n \
-  None<Key>4:		guess-meaning(3)\n \
-  None<Key>5:		guess-meaning(4)\n \
-  None<Key>6:		guess-meaning(5)\n \
-  None<Key>7:		guess-meaning(6)\n \
-  None<Key>8:		guess-meaning(7)\n \
-  None<Key>9:		guess-meaning(8)\n \
-  None<Key>c:		Accelerator(c)\n \
-  None<Key>e:		Accelerator(e)\n \
-  None<Key>k:		Accelerator(k)\n \
-  None<Key>l:		Accelerator(l)\n \
-  None<Key>m:		Accelerator(m)\n \
+  <Key>1:		guess-meaning(0)\n \
+  <Key>2:		guess-meaning(1)\n \
+  <Key>3:		guess-meaning(2)\n \
+  <Key>4:		guess-meaning(3)\n \
+  <Key>5:		guess-meaning(4)\n \
+  <Key>6:		guess-meaning(5)\n \
+  <Key>7:		guess-meaning(6)\n \
+  <Key>8:		guess-meaning(7)\n \
+  <Key>9:		guess-meaning(8)\n \
+  Shift<Key>c:		Accelerator(C)\n \
+  <Key>c:		Accelerator(c)\n \
+  Shift<Key>e:		Accelerator(E)\n \
+  <Key>e:		Accelerator(e)\n \
+  Shift<Key>k:		Accelerator(K)\n \
+  <Key>k:		Accelerator(k)\n \
+  Any<Key>l:		Accelerator(l)\n \
+  Shift<Key>m:		Accelerator(M)\n \
+  <Key>m:		Accelerator(m)\n \
+  <Key>n:		Accelerator(n)\n \
   Shift<Key>o:		Accelerator(O)\n \
-  None<Key>F13:		Accelerator(O)\n \
-  None<Key>o:		Accelerator(o)\n \
-  None<Key>p:		Accelerator(b)\n \
-  None<Key>u:		Accelerator(u)\n \
+  <Key>F13:		Accelerator(O)\n \
+  <Key>o:		Accelerator(o)\n \
+  <Key>p:		Accelerator(b)\n \
   Any<Key>s:		Accelerator(s)\n \
-  None<Key>F19:		Accelerator(s)\n \
+  Shift<Key>t:		Accelerator(T)\n \
+  <Key>u:		Accelerator(u)\n \
+  <Key>F19:		Accelerator(s)\n \
   Shift<Key>a:		Accelerator(S0)\n \
   Shift<Key>1:		Accelerator(S1)\n \
   Shift<Key>2:		Accelerator(S2)\n \
@@ -531,17 +620,28 @@ static char *guessAccel = " \
   Shift<Key>4:		Accelerator(S4)\n \
   Shift<Key>5:		Accelerator(S5)\n \
   Shift<Key>6:		Accelerator(S6)\n \
+  Shift<Key>7:		Accelerator(S7)\n \
+  Shift<Key>=:		Accelerator(S7)\n \
   Shift<Key>+:		Accelerator(S7)\n \
+  Shift<Key>KP_Add:	Accelerator(S7)\n \
   <Message>WM_PROTOCOLS: delete-window()\n \
   <Message>WM_DELETE_WINDOW: delete-window()\n \
   ";
-
+/* The 'Shift=' thing is a hack, because usually the '+' is actually
+ * the '=' key plus shift
+ */
 
 /* Accelerator()
  *	A jumping-off point for keyboard accelerators, as set above.
- *	Adding a case here, needs a line added in the resource, above.
+ *	This handles multiple widgets, so it can get messy.
+ *	WARNING: It seems you can only have a single accelerator table
+ *	for the ENTIRE PROGRAM. Not on a window-by-window basis.
+ *	We check 'w' for pointer to the window involved, if needed
+ *
+ *	Adding a new key in here also needs a line added in an Accel list,
+ *	like guessAccel, above.
  */
-void Accelerator(Widget w,XEvent *event,String *params,Cardinal *num_params){
+void Accelerator(Widget w, XEvent *event,String *params,Cardinal *num_params){
 	int int_store;
 	char *param = *params;
 	
@@ -552,6 +652,8 @@ void Accelerator(Widget w,XEvent *event,String *params,Cardinal *num_params){
 	if(*num_params >1){
 		printf("Warning: Accelerator functions take one argument, and one only\n");
 	}
+
+
 	switch(param[0]){
 		case 'b':
 			BackCallback(NULL,NULL,NULL);
@@ -559,18 +661,40 @@ void Accelerator(Widget w,XEvent *event,String *params,Cardinal *num_params){
 		case 'c':
 			cheatcallback(NULL,NULL,NULL);
 			break;
+		case 'C':
+			supercheat();
+			break;
 		case 'e':
 			ChangeMode(NULL,(XtPointer) GUESS_ENGLISH,NULL);
 			break;
+		case 'E':
+			ChangeQuestion(NULL,(XtPointer) GUESS_ENGLISH,NULL);
+			break;
 		case 'k':
 			ChangeMode(NULL,(XtPointer) GUESS_KANJI,NULL);
 			break;
+		case 'K':
+			ChangeQuestion(NULL,(XtPointer) GUESS_KANJI,NULL);
+			break;
 		case 'l':
-			MakeLog(NULL,(XtPointer) NULL,NULL);
+			LearnCallback(NULL,(XtPointer) NULL,NULL);
 			break;
 		case 'm':
 			ChangeMode(NULL,(XtPointer) GUESS_KANA,NULL);
 			break;
+		case 'n':
+			if(w==learn_popup){
+				LearnNewChar(w, (XtPointer)True,NULL);
+			}
+			break;
+		case 'N':
+			if(w==learn_popup){
+				LearnNewChar(w, False,NULL);
+			}
+			break;
+		case 'M':
+			ChangeQuestion(NULL,(XtPointer) GUESS_KANA,NULL);
+			break;
 		case 'o':
 			ordercallback(orderbutton,NULL,NULL);
 			break;
@@ -583,13 +707,29 @@ void Accelerator(Widget w,XEvent *event,String *params,Cardinal *num_params){
 		case 's':
 			SearchCallback(NULL,NULL,NULL);
 			break;
-		/* handle two-digit accelerator code */
+
+		/* handle two-digit accelerator code.
+		 * This is actually S[123456+]
+		 */
 		case 'S':
 			param+=1;
 			int_store = (*param)-'0';
 			GradeCallback(gradeButtons[int_store],
 				      (XtPointer) int_store,NULL);
-			break;				      
+			break;
+
+		case 'T':
+			TimerCallback(NULL,NULL,NULL);
+			break;
+		case 'x':
+			if(w== kanjiMissed){
+				ClearMissed();
+				break;
+			}
+			break;
+		default:
+			printf("DEBUG: unrecognized Accelerator '%c'\n",
+			       param[0]);
 	}
 }
 
@@ -622,12 +762,13 @@ delete_callback(Widget w, XEvent *event,String *params,Cardinal *num_params)
 
 void quitaction(Widget w, XEvent * e, String *p, Cardinal *c)
 {
-	puts("quitaction");
 	quit(NULL, NULL, NULL);
 }
+
 static XtActionsRec kdrillActionList[] = {
 	{ "quit",		quitaction},
 	{ "update-frequency",	UpdateFrequency},
+	{ "update-timeout",	UpdateTimeout},
 	{ "do-find",		DoFind},
 	{ "jump-to-kanji",	JumpToKanji},
 	{ "guess-meaning",	Guessvalue},
@@ -654,12 +795,19 @@ void MakeWidgets(){
 	/* I suppose I shouldn't hard-code window-width and height..
 		but it makes things easier for now.
 	*/
+
+#ifdef USEFORMFORTOP
 	form = XtVaCreateManagedWidget("kdrill",formWidgetClass,
 					   toplevel,
-			XtNx,200, XtNy,100,
 			XtNvSpace,16,
 			XtNhSpace,10,
 			NULL);
+#else
+
+	form = XtVaCreateManagedWidget("kdrill",panedWidgetClass,
+					   toplevel,
+			NULL);
+#endif
 
 	initfonts();
 
@@ -669,12 +817,14 @@ void MakeWidgets(){
 	/*XtOverrideTranslations(form,AllAccel);*/
 	XtAugmentTranslations(form,AllAccel);
 
+	MakeSpecialForms(form);
 
 	MakeQuestionWidget();
 	MakeChoicesButtons();
 
 	MakeOptionsPopup();
 	MakeSearchPopup();
+	MakeLearnPopup();
 	MakeMulti();
 	MakeButtons();
 
